"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.caseInsensitiveStrategy = exports.exactStrategy = exports.prefixedStrategy = exports.longShortStrategy = exports.longStrategy = exports.noStrategy = void 0;
/**
 * Do not match any unordered argument at all.
 * @returns The strategy.
 */
function noStrategy() {
    return {
        matchFlag: () => null,
        matchOption: () => null,
        matchCompactOption: () => null
    };
}
exports.noStrategy = noStrategy;
/**
 * Match unordered arguments according to conventional syntax.
 * '--flag' is a flag.
 * '--opt=' is an option.
 * '--opt=123' is a compact option.
 * @returns The strategy.
 */
function longStrategy() {
    return prefixedStrategy(['--'], ['=']);
}
exports.longStrategy = longStrategy;
/**
 * Match unordered arguments according to conventional syntax.
 * '--flag' or '-flag' is a flag.
 * '--opt=' or '-opt=' is an option.
 * '--opt=123' or '-opt=123' is a compact option.
 * @returns The strategy.
 */
function longShortStrategy() {
    return prefixedStrategy(['--', '-'], ['=']);
}
exports.longShortStrategy = longShortStrategy;
/**
 * Match unordered arguments with custom prefix and separator.
 * The prefix is the part the preceeds the key name, e.g. '--' in '--foo'.
 * The separator is the part that delimits the key and value e.g. '=' in '--key=value'.
 * It is expected that there are no spaces in the prefixes and separators.
 * The matching is done in a case-sensitive manner.
 * Also note that if the input contains multiple of the separators, the matching may be ambiguous.
 *
 * ```ts
 * const st = prefixedStrategy(['--'], ['=']);
 * console.log(st.matchFlag('--f'));
 * >>> 'f'
 *
 * console.log(st.matchOption('--opt='));
 * >>> 'opt'
 *
 * console.log(st.matchCompactOption('--opt=15'));
 * >>> ['opt', '15']
 * ```
 *
 * @param prefixes - The prefixes to use for unordered arguments.
 * They should be ordered by length in non-increasing order.
 * @param separators - The symbols to use to separate the key and value in options.
 * They should be ordered by length in non-increasing order.
 * @returns The strategy.
 */
function prefixedStrategy(prefixes, separators) {
    return {
        matchFlag(s) {
            const pre = prefixes.find(x => s.startsWith(x));
            if (pre == null) {
                return null;
            }
            s = s.slice(pre.length);
            const sep = separators.find(x => s.includes(x));
            if (sep != null) {
                return null;
            }
            return s;
        },
        matchOption(s) {
            const pre = prefixes.find(x => s.startsWith(x));
            if (pre == null) {
                return null;
            }
            s = s.slice(pre.length);
            const sep = separators.find(x => s.endsWith(x));
            if (sep == null) {
                return null;
            }
            return s.slice(0, -sep.length);
        },
        matchCompactOption(s) {
            const pre = prefixes.find(x => s.startsWith(x));
            if (pre == null) {
                return null;
            }
            s = s.slice(pre.length);
            const sep = separators.find(x => s.includes(x));
            if (sep == null) {
                return null;
            }
            const i = s.indexOf(sep);
            if (i + sep.length === s.length) {
                return null;
            }
            const k = s.slice(0, i);
            const v = s.slice(i + sep.length);
            return [k, v];
        }
    };
}
exports.prefixedStrategy = prefixedStrategy;
/**
 * Match unordered arguments according to a record of the names to the list of words in a case-sensitive manner.
 * Prefixes like '--' and separators like '=' should be a part of the word.
 * For case-insensitive matching, use {@linkcode caseInsensitiveStrategy}.
 *
 * ```ts
 * const st = exactStrategy({ flag: ['--flag', '-f'] }, {});
 * console.log(st.matchFlag('--flag'));
 * >>> 'flag'
 *
 * console.log(st.matchOption('-f'));
 * >>> 'flag'
 * ```
 *
 * @param flags - Words usable as flags.
 * @param options - Words usable as options.
 * They should be ordered by length in non-increasing order.
 * @returns The strategy.
 */
function exactStrategy(flags, options) {
    return {
        matchFlag(s) {
            var _a, _b;
            return (_b = (_a = Object.entries(flags)
                .find(xs => xs[1].some(x => s === x))) === null || _a === void 0 ? void 0 : _a[0]) !== null && _b !== void 0 ? _b : null;
        },
        matchOption(s) {
            var _a, _b;
            return (_b = (_a = Object.entries(options)
                .find(xs => xs[1].some(x => s === x))) === null || _a === void 0 ? void 0 : _a[0]) !== null && _b !== void 0 ? _b : null;
        },
        matchCompactOption(s) {
            var _a, _b;
            const k = (_b = (_a = Object.entries(options)
                .find(xs => xs[1].some(x => s.startsWith(x)))) === null || _a === void 0 ? void 0 : _a[0]) !== null && _b !== void 0 ? _b : null;
            if (k == null) {
                return null;
            }
            const v = s.slice(k.length);
            return [k, v];
        }
    };
}
exports.exactStrategy = exactStrategy;
/**
 * Match unordered arguments according to a record of the names to a list of words in a case-insensitive manner.
 * Prefixes like '--' and separators like '=' should be a part of the word.
 * For case-sensitive matching, use {@linkcode exactStrategy}.
 *
 * ```ts
 * const st = caseInsensitiveStrategy({ flag: ['--flag', '-f'] }, {});
 * console.log(st.matchFlag('--FlAg'));
 * >>> 'flag'
 *
 * console.log(st.matchOption('-F'));
 * >>> 'flag'
 * ```
 *
 * @param flags - Words usable as flags.
 * @param options - Words usable as options.
 * They should be ordered by length in non-increasing order.
 * @param locale - The locale(s) to use to compare case.
 * @returns The strategy.
 */
function caseInsensitiveStrategy(flags, options, locale) {
    return {
        matchFlag(s) {
            var _a, _b;
            s = s.toLocaleLowerCase(locale);
            return (_b = (_a = Object.entries(flags)
                .find(xs => xs[1].some(x => s === x.toLocaleLowerCase(locale)))) === null || _a === void 0 ? void 0 : _a[0]) !== null && _b !== void 0 ? _b : null;
        },
        matchOption(s) {
            var _a, _b;
            s = s.toLocaleLowerCase(locale);
            return (_b = (_a = Object.entries(options)
                .find(xs => xs[1].some(x => s === x.toLocaleLowerCase(locale)))) === null || _a === void 0 ? void 0 : _a[0]) !== null && _b !== void 0 ? _b : null;
        },
        matchCompactOption(s) {
            var _a, _b;
            const s1 = s.toLocaleLowerCase(locale);
            const k = (_b = (_a = Object.entries(options)
                .find(xs => xs[1].some(x => s1.startsWith(x.toLocaleLowerCase(locale))))) === null || _a === void 0 ? void 0 : _a[0]) !== null && _b !== void 0 ? _b : null;
            if (k == null) {
                return null;
            }
            const v = s.slice(k.length);
            return [k, v];
        }
    };
}
exports.caseInsensitiveStrategy = caseInsensitiveStrategy;
//# sourceMappingURL=unordered.js.map