/**
 * A strategy to match unordered arguments in parsing.
 */
export interface UnorderedStrategy {
    /**
     * Match a flag.
     * @param s - The string.
     * @returns The name of the flag.
     */
    matchFlag(s: string): string | null;
    /**
     * Match an option.
     * @param s - The string.
     * @returns The name of the option.
     */
    matchOption(s: string): string | null;
    /**
     * Match a compact option.
     * @param s - The string.
     * @returns A pair containing the name of the option and the value.
     */
    matchCompactOption(s: string): [string, string] | null;
}
/**
 * Do not match any unordered argument at all.
 * @returns The strategy.
 */
export declare function noStrategy(): UnorderedStrategy;
/**
 * Match unordered arguments according to conventional syntax.
 * '--flag' is a flag.
 * '--opt=' is an option.
 * '--opt=123' is a compact option.
 * @returns The strategy.
 */
export declare function longStrategy(): UnorderedStrategy;
/**
 * Match unordered arguments according to conventional syntax.
 * '--flag' or '-flag' is a flag.
 * '--opt=' or '-opt=' is an option.
 * '--opt=123' or '-opt=123' is a compact option.
 * @returns The strategy.
 */
export declare function longShortStrategy(): UnorderedStrategy;
/**
 * Match unordered arguments with custom prefix and separator.
 * The prefix is the part the preceeds the key name, e.g. '--' in '--foo'.
 * The separator is the part that delimits the key and value e.g. '=' in '--key=value'.
 * It is expected that there are no spaces in the prefixes and separators.
 * The matching is done in a case-sensitive manner.
 * Also note that if the input contains multiple of the separators, the matching may be ambiguous.
 *
 * ```ts
 * const st = prefixedStrategy(['--'], ['=']);
 * console.log(st.matchFlag('--f'));
 * >>> 'f'
 *
 * console.log(st.matchOption('--opt='));
 * >>> 'opt'
 *
 * console.log(st.matchCompactOption('--opt=15'));
 * >>> ['opt', '15']
 * ```
 *
 * @param prefixes - The prefixes to use for unordered arguments.
 * They should be ordered by length in non-increasing order.
 * @param separators - The symbols to use to separate the key and value in options.
 * They should be ordered by length in non-increasing order.
 * @returns The strategy.
 */
export declare function prefixedStrategy(prefixes: string[], separators: string[]): UnorderedStrategy;
/**
 * Pairing of flag/option names to the words usable for them.
 */
declare type Pairing = Record<string, string[]>;
/**
 * Match unordered arguments according to a record of the names to the list of words in a case-sensitive manner.
 * Prefixes like '--' and separators like '=' should be a part of the word.
 * For case-insensitive matching, use {@linkcode caseInsensitiveStrategy}.
 *
 * ```ts
 * const st = exactStrategy({ flag: ['--flag', '-f'] }, {});
 * console.log(st.matchFlag('--flag'));
 * >>> 'flag'
 *
 * console.log(st.matchOption('-f'));
 * >>> 'flag'
 * ```
 *
 * @param flags - Words usable as flags.
 * @param options - Words usable as options.
 * They should be ordered by length in non-increasing order.
 * @returns The strategy.
 */
export declare function exactStrategy(flags: Pairing, options: Pairing): UnorderedStrategy;
/**
 * Match unordered arguments according to a record of the names to a list of words in a case-insensitive manner.
 * Prefixes like '--' and separators like '=' should be a part of the word.
 * For case-sensitive matching, use {@linkcode exactStrategy}.
 *
 * ```ts
 * const st = caseInsensitiveStrategy({ flag: ['--flag', '-f'] }, {});
 * console.log(st.matchFlag('--FlAg'));
 * >>> 'flag'
 *
 * console.log(st.matchOption('-F'));
 * >>> 'flag'
 * ```
 *
 * @param flags - Words usable as flags.
 * @param options - Words usable as options.
 * They should be ordered by length in non-increasing order.
 * @param locale - The locale(s) to use to compare case.
 * @returns The strategy.
 */
export declare function caseInsensitiveStrategy(flags: Pairing, options: Pairing, locale?: string | string[]): UnorderedStrategy;
export {};
